      program jimslj

      implicit none

      include 'param.f'

c in/out
      integer ntot0,ntot1,nn,initrng,natom,is,nsamp,
     &    nr,ir,mnsamp,imin,ifail,nn0,nn1,ranseed
      character*16 filex0,filex1
      character*2 dum,symb0(mnat),symb1(mnat),symb(mnat),symbz(mnat)
      parameter(mnsamp=50000)
      double precision xx(3,mnat),mm0(mnat),mm1(mnat)
      double precision xxz(3,mnat)
      double precision xx0(mnsamp,3,mnat)
      double precision xx1(mnsamp,3,mnat)
      double precision mtot,xtot,ytot,ztot,v,vp,vm,reps,gv,rdiff,hv
      double precision x(mnat),y(mnat),z(mnat)
      double precision dx(mnat),dy(mnat),dz(mnat)
      double precision vmin,vmax,vavg,vstd
      double precision rrmin,rrmax,rravg,rrstd,rrlast
      double precision ssmin,ssmax,ssavg,ssstd
      double precision rmin,rmax,xr,xv,zero
      integer ncalc1,ncalc2
      logical lfail,ldebug

c local
      integer n,i,j,nrcl,itmp1,itmp2
      double precision rr,r

      double precision tmpprint(50)
      common/tmp/tmpprint

c HEADER
      write(6,*)
      write(6,*)"---------------ONE D MIN---------------"
      write(6,*)" A code for calculating Lennard-Jones parameters",
     & " from detailed intermolecular"
      write(6,*)" potentials via one-dimensional minimizations, ",
     & "as described in "
      write(6,*)
      write(6,*)" A. W. Jasper and J. A. Miller, Combust. Flame ",
     & "161, 101 (2014)"
      write(6,*)
      call prepot
      write(6,*)"---------------------------------------"
      write(6,*)

c READ INPUT
      read(5,*)ranseed
      read(5,*)ntot0,filex0
      read(5,*)ntot1,filex1
      read(5,*)nsamp
      read(5,*)rmin,rmax
      read(5,*)zero
c      ldebug = .true.  ! TURN ON DEBUG PRINTING
      ldebug = .false.

      write(6,*)"-----------------INPUT-----------------"
      write(6,*)'RANSEED = ',ranseed
      write(6,*)'TARGET GEOMS: NUMBER = ',ntot0,', FILE = ',filex0
      write(6,*)'BATH GEOMS:   NUMBER = ',ntot1,', FILE = ',filex1
      write(6,*)'SAMPLES = ',nsamp
      write(6,*)'C.O.M. range = ',rmin,rmax,' A'
      write(6,*)'ZERO = ',zero,' Eh'
      write(6,*)"---------------------------------------"
      write(6,*)

      write(6,*)"-----------------OUTPUT----------------"

! initialize RNG
      call srand(ranseed)

! dump XYZ file for the target into an array
      open(unit=93,file=filex0)
      read(93,*)
      read(93,*)
      do i=1,ntot0
      read(93,*)nn0
      read(93,*)
      do j=1,nn0
        read(93,*)symb0(j),xx0(i,1,j),xx0(i,2,j),xx0(i,3,j)
        xx0(i,1,j)=xx0(i,1,j)/autoang
        xx0(i,2,j)=xx0(i,2,j)/autoang
        xx0(i,3,j)=xx0(i,3,j)/autoang
        call mass(symb0(j),mm0(j))
      enddo
      enddo
      close(93)

! dump XYZ file for the bath into an array
      open(unit=93,file=filex1)
      read(93,*)
      read(93,*)
      do i=1,ntot1
      read(93,*)nn1
      read(93,*)
      do j=1,nn1
        read(93,*)symb1(j),xx1(i,1,j),xx1(i,2,j),xx1(i,3,j)
c        print *,symb1(j),xx1(i,1,j),xx1(i,2,j),xx1(i,3,j)
        xx1(i,1,j)=xx1(i,1,j)/autoang
        xx1(i,2,j)=xx1(i,2,j)/autoang
        xx1(i,3,j)=xx1(i,3,j)/autoang
        call mass(symb1(j),mm1(j))
        if (symb1(j).eq."H".or.symb1(j).eq."h") symb1(j)="H2"
        if (symb1(j).eq."N".or.symb1(j).eq."n") symb1(j)="N2"
        if (symb1(j).eq."O".or.symb1(j).eq."o") symb1(j)="O3" !!  important to use "O3" not "O2"
      enddo
      enddo
      close(93)

! INITIALIZE
      vavg=0.d0
      vstd=0.d0
      vmax=-10000.d0
      vmin=0.d0
      ssmax=0.d0
      ssmin=100.d0
      rrmax=0.d0
      rrmin=100.d0
      rravg=0.d0
      ssavg=0.d0
      rrstd=0.d0
      ssstd=0.d0

      write(6,'(a10,a15,a7,a12,a19,a13,a15,a17)')
     & "n","NR steps: min","inner",
     & "r_n, A",
     & "2^-1/6 r_n, A",
     & "V_n, cm-1",
     & "sig_n, A","V_inner, cm-1"

      do is = 1,nsamp    ! Loop over N

      ncalc1=0
      ncalc2=0
      rr=(rmax+rmin)/2.d0

! Sample and spin the bath
      r=dble(rand())
      r=r*dble(ntot1)
      n=int(r)+1
      do j=1,nn1
        xx(1,j)=xx1(n,1,j)
        xx(2,j)=xx1(n,2,j)
        xx(3,j)=xx1(n,3,j)
      enddo
      xtot=0.d0
      ytot=0.d0
      ztot=0.d0
      mtot=0.d0
      do i=1,nn1
          xtot = xtot + mm1(i)*xx(1,i)
          ytot = ytot + mm1(i)*xx(2,i)
          ztot = ztot + mm1(i)*xx(3,i)
          mtot = mtot + mm1(i)
      enddo
      do i=1,nn1
          xx(1,i) = xx(1,i) - xtot/mtot
          xx(2,i) = xx(2,i) - ytot/mtot
          xx(3,i) = xx(3,i) - ztot/mtot
      enddo
      call spin(xx,nn1)

! move the bath to the end of the xx array
      do i=1,nn1
         xxz(1,i+nn0)=xx(1,i)
         xxz(2,i+nn0)=xx(2,i)
         xxz(3,i+nn0)=xx(3,i)
         symbz(i+nn0)=symb1(i)
      enddo
      do i=1,nn1
         xx(1,i+nn0)=xxz(1,i+nn0)
         xx(2,i+nn0)=xxz(2,i+nn0)
         xx(3,i+nn0)=xxz(3,i+nn0)
         symb(i+nn0)=symbz(i+nn0)
      enddo

! Sample and spin the target
      r=dble(rand())
      r=r*dble(ntot0)
      n=int(r)+1
      do j=1,nn0
        xx(1,j)=xx0(n,1,j)
        xx(2,j)=xx0(n,2,j)
        xx(3,j)=xx0(n,3,j)
      enddo
      xtot=0.d0
      ytot=0.d0
      ztot=0.d0
      mtot=0.d0
      do i=1,nn0
          xtot = xtot + mm0(i)*xx(1,i)
          ytot = ytot + mm0(i)*xx(2,i)
          ztot = ztot + mm0(i)*xx(3,i)
          mtot = mtot + mm0(i)
      enddo
      do i=1,nn0
          xx(1,i) = xx(1,i) - xtot/mtot
          xx(2,i) = xx(2,i) - ytot/mtot
          xx(3,i) = xx(3,i) - ztot/mtot
      enddo
      call spin(xx,nn0)

      do i=1,nn0
         symb(i)=symb0(i)
      enddo

      ifail=0
 10   continue
      lfail=.false.

c NR search for minimum energy
      rdiff = 100.
      do while (rdiff > .001d0)   ! Convergence tolerance of CoM in A
      ncalc1=ncalc1+1
      reps = 0.001d0 ! Numerical gradient and hessian parameter

      nn=nn0+nn1

      do i=1,nn0
      x(i) = xx(1,i)
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo

      do i=nn0+1,nn
      x(i) = xx(1,i)+(rr+reps)/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,vp,dx,dy,dz,nn,nn)
      vp=(tmpprint(1)-zero)*autocmi

      if (ldebug) then
        write(80,*)nn
        write(80,*)is,rr,v
        do i=1,nn
        write(80,*)symb(i),x(i)*autoang,y(i)*autoang,z(i)*autoang
        enddo
      endif

      do i=nn0+1,nn
      x(i) = xx(1,i)+(rr-reps)/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,vm,dx,dy,dz,nn,nn)
      vm=(tmpprint(1)-zero)*autocmi

      do i=nn0+1,nn
      x(i) = xx(1,i)+rr/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,v,dx,dy,dz,nn,nn)
      v=(tmpprint(1)-zero)*autocmi

      gv=(vp-vm)/(2.d0*reps)  ! numerical gradient w.r.t. the CoM distance
      hv=(vp+vm-2.d0*v)/(reps**2)  ! numerical hessian w.r.t. the CoM distance

      rrlast=rr
      rr=rr-gv/hv   ! optimize to minimum
      rdiff=abs(gv/hv)

      if (ldebug) write(6,*)'r',ncalc1,rr,rrlast,v,vp,vm

      if (rr.gt.rmax.or.rr.ne.rr) then 
         ifail=ifail+1
         lfail=.true.
      elseif (rr.lt.rmin) then 
         ifail=ifail+1
         lfail=.true.
      elseif (v.lt.-5000) then 
         ifail=ifail+1
         lfail=.true.
      elseif (v.gt.5000) then 
         ifail=ifail+1
         lfail=.true.
c      elseif (ncalc1.gt.100) then 
c         ifail=ifail+1
c         lfail=.true.
      endif
      if (lfail) then
      lfail=.false.
      if (ifail.gt.20) then
         print *,'20 fails'
         stop
      elseif(ifail.gt.0.and.ifail.lt.10) then
         rr=rmax-dble(ifail-1)*(rmax-rmin)/dble(20)
         go to 10
      elseif(ifail.gt.9.and.ifail.lt.20) then
         rr=rmax-dble(ifail)*(rmax-rmin)/dble(20)
         go to 10
      endif
      endif

      enddo

      rrmax=max(rrmax,rr)
      rrmin=min(rrmin,rr)
      rravg=rravg+rr
      rrstd=rrstd+rr**2
      vmax=max(vmax,v)
      vmin=min(vmin,v)
      vavg=vavg+v
      vstd=vstd+v**2

      xr=rr
      xv=v

c      if (ldebug) then
c        write(80,*)nn
c        write(80,*)is,rr,v
c        do i=1,nn
c        write(80,*)symb(i),x(i)*autoang,y(i)*autoang,z(i)*autoang
c        enddo
c      endif

! NR search for inner turning point
      rr=rr/(2.d0**(1.d0/6.d0))

      ifail=0
 11   continue
      lfail=.false.

      rdiff = 100.
      do while (rdiff > .001d0) 
      ncalc2=ncalc2+1
      reps = 0.0001d0

      do i=1,nn0
      x(i) = xx(1,i)
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      do i=nn0+1,nn
      x(i) = xx(1,i)+rr/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,v,dx,dy,dz,nn,nn)
      v=(tmpprint(1)-zero)*autocmi

      if (ldebug) then
        write(80,*)nn
        write(80,*)is,rr,v
        do i=1,nn
        write(80,*)symb(i),x(i)*autoang,y(i)*autoang,z(i)*autoang
        enddo
      endif

      do i=nn0+1,nn
      x(i) = xx(1,i)+(rr+reps)/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,vp,dx,dy,dz,nn,nn)
      vp=(tmpprint(1)-zero)*autocmi

      do i=nn0+1,nn
      x(i) = xx(1,i)+(rr-reps)/autoang
      y(i) = xx(2,i)
      z(i) = xx(3,i)
      enddo
      call pot(symb,x,y,z,vm,dx,dy,dz,nn,nn)
      vm=(tmpprint(1)-zero)*autocmi

      gv=(vp-vm)/(2.d0*reps)  ! grad
      hv=(vp+vm-2.d0*v)/(reps**2)  ! hess

      rrlast=rr
      rr=rr-v/gv   ! optimize to inner turning point
      rdiff=abs(v/gv)

      if (ldebug) write(6,*)'s',ncalc2,rr,rrlast,v,vp,vm

      if (rr.gt.rmax.or.rr.ne.rr) then 
         ifail=ifail+1
         lfail=.true.
      elseif (rr.lt.rmin) then 
         ifail=ifail+1
         lfail=.true.
      elseif (v.lt.-5000) then 
         ifail=ifail+1
         lfail=.true.
      elseif (v.gt.5000) then 
         ifail=ifail+1
         lfail=.true.
c      elseif (ncalc2.gt.100) then 
c         ifail=ifail+1
c         lfail=.true.
      endif
      if (lfail) then
      lfail=.false.
      if (ifail.gt.10) then
         print *,'10 fails'
         stop
      elseif(ifail.gt.0.and.ifail.lt.5) then
         rr=rmax-dble(ifail-1)*(rmax-rmin)/dble(10)
         go to 11
      elseif(ifail.gt.4.and.ifail.lt.10) then
         rr=rmax-dble(ifail)*(rmax-rmin)/dble(10)
         go to 11
      endif
      endif

      enddo

      ssmax=max(ssmax,rr)
      ssmin=min(ssmin,rr)
      ssavg=ssavg+rr
      ssstd=ssstd+rr**2

      write(6,'(3i10,20f15.5)')is,ncalc1,ncalc2,xr,
     & xr/(2.d0**(1.d0/6.d0)),xv,rr,v

      enddo

      vstd=dsqrt(dble(nsamp)*vstd-vavg**2)/dble(nsamp)
      vavg=vavg/dble(nsamp)
      rrstd=dsqrt(dble(nsamp)*rrstd-rravg**2)/dble(nsamp)
      rravg=rravg/dble(nsamp)
      ssstd=dsqrt(dble(nsamp)*ssstd-ssavg**2)/dble(nsamp)
      ssavg=ssavg/dble(nsamp)

      write(6,*)
      write(6,'(a27,3x,20f15.5)')'minimum    ',rrmin,
     & rrmin/(2.d0**(1.d0/6.d0)),
     & vmin,ssmin
      write(6,'(a27,3x,20f15.5)')'maximum    ',rrmax,
     & rrmax/(2.d0**(1.d0/6.d0)),
     & vmax,ssmax
      write(6,'(a27,3x,20f15.5)')'average    ',rravg,
     & rravg/(2.d0**(1.d0/6.d0)),
     & vavg,ssavg
      write(6,'(a27,3x,20f15.5)')'standard dev. ',rrstd,
     & rrstd/(2.d0**(1.d0/6.d0)),
     & vstd,ssstd

      write(6,*)
      write(6,*)"    SIGMA  = ",ssavg," +/- ",ssstd," A"
      write(6,*)"   EPSILON = ",vavg," +/- ",vstd," cm-1"
      write(6,*)
c      write(6,*)ssavg,ssstd,vavg,vstd

      write(6,*)"---------------------------------------"
      write(6,*)"Well done!"

      end

      subroutine mass(x,y)

      implicit none

      character*2 x
      double precision y

! ADD MASSES HERE!!
      y=0.d0
      if (x.eq."H".or.x.eq."h") y= 1.007825d0
      if (x.eq."C".or.x.eq."c") y=12.d0
      if (x.eq."O".or.x.eq."o") y=15.994915d0
      if (x.eq."N".or.x.eq."n") y=14.003074d0
      if (x.eq."He".or.x.eq."he".or.x.eq."HE") y=4.002603d0
      if (x.eq."Ne".or.x.eq."ne".or.x.eq."NE") y=19.992440d0
      if (x.eq."Ar".or.x.eq."ar".or.x.eq."AR") y=39.962383d0
      if (x.eq."Kr".or.x.eq."kr".or.x.eq."KR") y=83.911507d0
      if (x.eq."Si".or.x.eq."si".or.x.eq."SI") y=27.9769265d0
      if (y.eq.0.d0) then
        write(6,*)"Don't know the mass of ",x
        write(6,*)"Please add it to onedmin.F in the subroutine mass"
        stop
      endif

      return

      end

